#ifndef __CPHIDGET
#define __CPHIDGET

typedef struct _CPhidget CPhidget, *CPhidgetHandle;

#ifndef EXTERNALPROTO
#include "cphidgetattr.h"
#include "cphidgetconstants.h"
#include "cthread.h"

#ifdef _WINDOWS
#ifdef __GNUC__
typedef UINT_PTR uintptr_t;
#endif
#endif

//This is for a socket - an one of these can be referenced by several phidgets!
typedef struct __CPhidgetSocketClient {
	int socket;
	char *port;
	char *address;
	char *serverID;
	void *pdcs;
	int listen_id;
	int status;
	CThread_mutex_t lock; /* protects server status */
	CThread_mutex_t pdc_lock; /* protects synchronous pcd functions */
} CPhidgetSocketClient, *CPhidgetSocketClientHandle;

typedef struct __CPhidgetSocketClientList
{
	struct __CPhidgetSocketClientList *next;
	CPhidgetSocketClientHandle client;
} CPhidgetSocketClientList, *CPhidgetSocketClientListHandle;

struct _CPhidget {
	CPhidgetSocketClientHandle networkInfo;
	char *password;
	int(CCONV *fptrError)(CPhidgetHandle , void *, int, const char *);
	void *fptrErrorptr;
	int listen_id;
	CThread_mutex_t lock; /* protects status */
	int status;
	CThread_mutex_t writelock; /* protects write - exclusive */
	int keyCount; //counts key during network open
	CThread readThread;
	CThread writeThread;
	int specificDevice;
	unsigned short deviceID;
	unsigned short deviceIDSpec;
	unsigned short outputReportByteLength;
	unsigned short inputReportByteLength;
	HANDLE deviceHandle;
	EVENT closeReadEvent;
	EVENT closeWriteEvent;
	const char *deviceType;
	int deviceVersion;
	unsigned short ProductID;
	unsigned short VendorID;
	int serialNumber;
	int(CCONV *fptrInit)(CPhidgetHandle, unsigned char *, int length);
	int(CCONV *fptrData)(CPhidgetHandle, unsigned char *buffer, int length);
	int(CCONV *fptrMakePacket)(CPhidgetHandle, unsigned char *buffer,
	  unsigned int *length);
	unsigned char lastReadPacket[8];
	//char	ascii_representation[89];
	char label[11];
	unsigned char awdc_enabled;
	void *CPhidgetFHandle;
	CThread_mutex_t outputLock; /* device-specific code responsible */
	EVENT writeAvailableEvent; /* device-specific code sets w/o OLL held */
	EVENT writtenEvent; /* device-specific code clears w/OLL held */
	int writeStopFlag; /* set when closing */
	int(CCONV *fptrAttach)(CPhidgetHandle , void *);
	void *fptrAttachptr; 
	int(CCONV *fptrDetach)(CPhidgetHandle , void *);
	void *fptrDetachptr;
	CPhidgetAttr attr;
};

struct _CPhidgetList
{
	struct _CPhidgetList *next;
	CPhidgetHandle phid;
} typedef CPhidgetList, *CPhidgetListHandle;

typedef struct {
	int pdd_did;
	int pdd_vid;
	int pdd_pid;
	CPhidgetAttr pdd_attr;
	int pdd_iid;
} CPhidgetDeviceDef;


extern CPhidgetList *ActiveDevices;
extern CPhidgetList *AttachedDevices;
extern int phidgetLockInitialized;
extern CThread_mutex_t phidgetLock;

extern const CPhidgetDeviceDef Phid_Device_Def[];

int CPhidget_read(CPhidgetHandle phid);
int CPhidget_write(CPhidgetHandle phid);
int CPhidget_isAttached(CPhidgetHandle phid);
void CPhidget_setStatus(CPhidgetHandle phid, int v);
char translate_bool_to_ascii(char value);
const char *CPhidget_strerror(int error);
void throw_error_event(CPhidgetHandle phid, const char *error, int errcode);

PHIDGET21_API int CCONV CPhidget_areEqual(void *arg1, void *arg2);
PHIDGET21_API int CCONV CPhidgetHandle_areEqual(void *arg1, void *arg2);
PHIDGET21_API void CCONV CPhidget_free(void *arg);
PHIDGET21_API int CCONV phidget_name_to_id(const char *name);
PHIDGET21_API int CCONV CPhidget_create(CPhidgetHandle *phid);

PHIDGET21_API int CCONV CPhidget_calibrate_gainoffset(CPhidgetHandle phid, int Index, unsigned short offset, unsigned long gain);

#endif
#include "cphidgetmacros.h"

PHIDGET21_API int CCONV CPhidget_open(CPhidgetHandle phid, int serialNumber);
PHIDGET21_API int CCONV CPhidget_close(CPhidgetHandle phid);
PHIDGET21_API int CCONV CPhidget_delete(CPhidgetHandle phid);
PHIDGET21_API int CCONV CPhidget_set_OnDetach_Handler(CPhidgetHandle,
    int(CCONV *fptr)(CPhidgetHandle, void *), void *userPtr);
PHIDGET21_API int CCONV CPhidget_set_OnAttach_Handler(CPhidgetHandle, int(CCONV *fptr)(CPhidgetHandle, void *), void *userPtr);
PHIDGET21_API int CCONV CPhidget_set_OnError_Handler(CPhidgetHandle phid, int(CCONV *fptr)(CPhidgetHandle, void *, int, const char *), void *userPtr);
PHIDGET21_API int CCONV CPhidget_getDeviceName(CPhidgetHandle phid,
    const char **buffer);
PHIDGET21_API int CCONV CPhidget_getSerialNumber(CPhidgetHandle phid,
    int *serialNumber);
PHIDGET21_API int CCONV CPhidget_getDeviceVersion(CPhidgetHandle phid,
    int *devVer);
PHIDGET21_API int CCONV CPhidget_getDeviceStatus(CPhidgetHandle phid,
    int *status);
PHIDGET21_API int CCONV CPhidget_getLibraryVersion(const char **buffer);
PHIDGET21_API int CCONV CPhidget_getDeviceType(CPhidgetHandle phid,
    const char **buffer);
PHIDGET21_API int CCONV CPhidget_getDeviceLabel(CPhidgetHandle phid, const char **buffer);
PHIDGET21_API int CCONV CPhidget_setDeviceLabel(CPhidgetHandle phid, const char *buffer);
PHIDGET21_API int CCONV CPhidget_getErrorDescription(int ErrorCode, const char **buf);
PHIDGET21_API int CCONV CPhidget_waitForAttachment(CPhidgetHandle phid, int milliseconds);
PHIDGET21_API int CCONV CPhidget_getServerID(CPhidgetHandle phid, const char **serverID);
PHIDGET21_API int CCONV CPhidget_getServerAddress(CPhidgetHandle phid, const char **address, int *port);
PHIDGET21_API int CCONV CPhidget_getServerStatus(CPhidgetHandle phid, int *status);

#endif
