#include "stdafx.h"
#include "cphidgetrfid.h"
#include "stdio.h"
#include "cusb.h"
#include "csocket.h"
#include "cthread.h"

static int CCONV CPhidgetRFID_makePacket(CPhidgetHandle phidG,
    unsigned char *buf, unsigned int *lenp)
{
	CPhidgetRFIDHandle phid = (CPhidgetRFIDHandle)phidG;

	/* check for tag lost */
#ifdef _WINDOWS
	TIME now;
	FILETIME nowft, oldft, resft;
	ULARGE_INTEGER nowul, oldul, resul;
	double duration;
	GetSystemTime(&now);
	SystemTimeToFileTime(&now, &nowft);
	SystemTimeToFileTime(&phid->lastTagTime, &oldft);

	nowul.HighPart = nowft.dwHighDateTime;
	nowul.LowPart = nowft.dwLowDateTime;
	oldul.HighPart = oldft.dwHighDateTime;
	oldul.LowPart = oldft.dwLowDateTime;

	resul.HighPart = nowul.HighPart - oldul.HighPart;
	resul.LowPart = nowul.LowPart - oldul.LowPart;

	resft.dwHighDateTime = resul.HighPart;
	resft.dwLowDateTime = resul.LowPart;

	duration = (double)(resft.dwLowDateTime/10000000.0);
#else
	struct timeval now;
	gettimeofday(&now, NULL);
	double duration = (now.tv_sec - phid->lastTagTime.tv_sec) + (double)((now.tv_usec-phid->lastTagTime.tv_usec)/1000000.0);
#endif
	if (phid->tagPresent && duration > 0.2) {
			phid->tagPresent = 0;
			if (phid->fptrTagLost)
				phid->fptrTagLost(phid, phid->fptrTagLostptr, phid->lastTag);
	}

	CThread_mutex_lock(&phid->phid.outputLock);
	*lenp = 1;
	memcpy(buf, phid->outputPacket, *lenp);
	CThread_mutex_unlock(&phid->phid.outputLock);
	return EPHIDGET_OK;
}

static int CCONV _sendpacket(CPhidgetRFIDHandle phid,
    const unsigned char *buf, unsigned int len)
{
	if (!CPhidget_isAttached(&phid->phid))
		return EPHIDGET_NOTATTACHED;
	CThread_mutex_lock(&phid->phid.outputLock);
	memcpy(phid->outputPacket, buf, len);
	CThread_reset_event(&phid->phid.writtenEvent);
	CThread_mutex_unlock(&phid->phid.outputLock);
	CThread_set_event(&phid->phid.writeAvailableEvent);
	return 0;
}

CFHANDLE(RFID,Output) int, int), void *userPtr)
ASGNFPTR(RFID,Output)

PHIDGET21_API int CCONV CPhidgetRFID_set_OnTag_Handler(CPhidgetRFIDHandle phid, int (CCONV *fptr)(CPhidgetRFIDHandle phid, void *, unsigned char *), void *userPtr)
{
	TESTPTR(phid)
	phid->fptrTag = fptr;
	phid->fptrTagptr = userPtr;
	return EPHIDGET_OK;
}

PHIDGET21_API int CCONV CPhidgetRFID_set_OnTagLost_Handler(CPhidgetRFIDHandle phid, int (CCONV *fptr)(CPhidgetRFIDHandle phid, void *, unsigned char *), void *userPtr)
{
	TESTPTR(phid)
	phid->fptrTagLost = fptr;
	phid->fptrTagLostptr = userPtr;
	return EPHIDGET_OK;
}

CPHIDGETDATA(RFID)
	unsigned char outbuf[8];
	int i, j, tag=0;
	TESTPTR(phid)

	if (buffer[0] == 0)
	{
#ifdef _WINDOWS
		GetSystemTime(&phid->lastTagTime);
#else
		gettimeofday(&phid->lastTagTime,NULL);
#endif
		if(strncmp((const char *)(buffer+1), (const char *)phid->lastTag, 5))
			phid->tagPresent = 0;
		phid->lastTag[0] = outbuf[0] = buffer[1];
		phid->lastTag[1] = outbuf[1] = buffer[2];
		phid->lastTag[2] = outbuf[2] = buffer[3];
		phid->lastTag[3] = outbuf[3] = buffer[4];
		phid->lastTag[4] = outbuf[4] = buffer[5];
		phid->lastTag[5] = outbuf[5] = outbuf[6] = outbuf[7] = 0;
		tag=1;
		if(!phid->tagPresent) {
			if (phid->fptrTag)
				phid->fptrTag(phid, phid->fptrTagptr, outbuf);
			phid->tagPresent = 1;
		}
	}
	else if (buffer[0] == 1)
	{
		if (phid->fullStateEcho)
		{
			for (i = 0, j = 0x01; i<4; i++, j = j * 2)
			{
				if ((buffer[1] & j) == 0)
				{
					if (phid->outputStates[i] == PUNK)
						phid->outputStates[i] = PFALSE;
					if (phid->outputEchoStates[i] != PFALSE)
					{
						phid->outputEchoStates[i] = PFALSE;
						FIRE(Output) i, PFALSE);
					}
				}
				else
				{
					if (phid->outputStates[i] == PUNK)
						phid->outputStates[i] = PTRUE;
					if (phid->outputEchoStates[i] != PTRUE)
					{	
						phid->outputEchoStates[i] = PTRUE;
						FIRE(Output) i, PTRUE);
					}
				}	
			}
		}
	}
	else
		return EPHIDGET_UNEXPECTED;
	
	return EPHIDGET_OK;
}

CGETINDEX(RFID,OutputState,int)
	TESTPTRS(phid,pVal) 
	TESTRNG1(phid.attr.rfid.numOutputs+2)
	MASGN1(outputStates[Index])
}

CSETINDEX(RFID,OutputState,int)
	int i,j,k,ret;
	unsigned char buffer[20];
	TESTPTR(phid)
	TESTRNG1(phid.attr.rfid.numOutputs+2)
	
	if(phid->phid.networkInfo)
		ADDNETWORKKEYINDEXED(Output, "%d")
	else
	{
		CThread_mutex_lock(&phid->phid.writelock);
		phid->outputStates[Index] = (unsigned char)newVal;

		for (i = 0; i<20; i++) buffer[i] = 0;

		if (CPhidget_isAttached(&phid->phid))
		{
			switch(phid->phid.ProductID)
			{
				case 0x0031:
					for (k = 0, j = 1; k<4; k++, j *= 2)
					{
						if (phid->outputStates[k] == PUNK)
							phid->outputStates[k] = PFALSE;

						if ((!(phid->fullStateEcho)) && (phid->outputEchoStates[k] == PUNK))
							phid->outputEchoStates[k] = phid->outputStates[k];
						if (phid->outputStates[k]) buffer[0] |= j;
					}
					ret = _sendpacket(phid, buffer, 1);
					CThread_mutex_unlock(&phid->phid.writelock);
					return ret;
				default:
					break;
			}
		}
		CThread_mutex_unlock(&phid->phid.writelock);
	}
	return EPHIDGET_UNEXPECTED;
}

CGET(RFID,NumOutputs,int)
	TESTPTRS(phid,pVal) 
	MASGN1(phid.attr.rfid.numOutputs)
}

CGET(RFID,AntennaOn,int)
	TESTPTRS(phid,pVal) 
	MASGN1(outputStates[3])
}

CGET(RFID,LEDOn,int)
	TESTPTRS(phid,pVal) 
	MASGN1(outputStates[2])
}

CSET(RFID,AntennaOn,int)
	TESTPTR(phid)
	if(phid->phid.networkInfo)
		ADDNETWORKKEY(AntennaOn, "%d")
	else
		CPhidgetRFID_setOutputState(phid, 3, (unsigned char)newVal);
	return EPHIDGET_OK;
}

CSET(RFID,LEDOn,int)
	TESTPTR(phid)
	if(phid->phid.networkInfo)
		ADDNETWORKKEY(LEDOn, "%d")
	else
		CPhidgetRFID_setOutputState(phid, 2, (unsigned char)newVal);
	return EPHIDGET_OK;
}

PHIDGET21_API int CCONV CPhidgetRFID_getLastTag(CPhidgetRFIDHandle phid, unsigned char *pVal)
{
	int i;
	TESTPTR(phid)
	TESTPTR(pVal)
	for (i = 0; i<5; i++)
		if (phid->lastTag[i]) break;
	if (i == 5)
		/* lastTag is not yet initialized */
		return EPHIDGET_UNKNOWNVAL;

	for (i = 0; i<5; i++)
		pVal[i] = phid->lastTag[i];
	pVal[5] = 0;
	return EPHIDGET_OK;
}

CPHIDGETINIT(RFID)
	int i;
	phid->phid.awdc_enabled = PTRUE; /* enable this to get the taglost events */
	if (phid->phid.deviceVersion >= 201)
		phid->fullStateEcho = PTRUE;
	for (i = 0; i<4; i++)
	{
		phid->outputStates[i] = PUNK;		/* State is unknown */
		phid->outputEchoStates[i] = PUNK;	/* State is unknown */
	}
	return EPHIDGET_OK;
}

CCREATE(RFID, PHIDDEF_RFID)
