#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <unistd.h>
#include <signal.h>
#include "pdict.h"

static int cl_ok = 0;
#ifndef _WINDOWS
static int should_exit = 0;
#endif
static int additions = 0;
static int changes = 0;

static void
cl(const char *k, const char *v, pdict_reason_t r, const char *oval,
    void *arg)
{
	assert(arg == (void *)1);
	if (strcmp(v, "value1b") == 0 && strcmp(oval, "value1") == 0 &&
	    (int)arg == 1)
		cl_ok = 1;
	changes++;
}

static void
count_additions(const char *k, const char *v, pdict_reason_t reason,
    const char *oval, void *arg)
{
	assert((int)arg == 0);
	if (reason == PDR_ENTRY_ADDED)
		additions++;
}

static int
print_entries(const char *k, const char *v, void *arg)
{
	printf("\t%s: %s\n", k, v);
	return 1;
}

#ifndef _WINDOWS
static void
sigalrm(int sig)
{
	should_exit = 1;
}
#endif

int
main(void)
{
#ifndef _WINDOWS
	int period = 10;
#endif
	const char *v;
	char *ov;
	pdict_t *pd;
	int id;
	
	if (!(pd = pdict_alloc())) {
		perror("pdict_alloc");
		return (1);
	}
	assert(id = pdict_add_persistent_change_listener(pd, ".", count_additions,
	    0));
	assert(pdict_add(pd, "key1", "value1", NULL));
	v = NULL;
	assert(pdict_ent_lookup(pd, "key1", &v));
	assert(v);
	assert(strcmp("value1", v) == 0);
	free((void *)v);

	assert(pdict_add(pd, "key1", "value1", NULL));
	assert(pdict_ent_lookup(pd, "key1", &v));
	assert(strcmp("value1", v) == 0);
	free((void *)v);

	assert(pdict_add(pd, "key1", "value2", NULL));
	assert(pdict_ent_lookup(pd, "key1", &v));
	assert(strcmp("value2", v) == 0);
	free((void *)v);

	assert(pdict_add(pd, "key1", "value1", NULL));
	assert(pdict_ent_lookup(pd, "key1", &v));
	assert(strcmp("value1", v) == 0);
	free((void *)v);

	assert(pdict_add(pd, "key2", "value2", NULL));

	assert(pdict_ent_lookup(pd, "key1", NULL));
	assert(pdict_ent_add_change_listener(pd, "key1", cl, (void *)1));
	assert(pdict_add(pd, "key1", "value1b", NULL));
	assert(cl_ok);
	printf("The dictionary has:\n");
	pdict_walk(pd, print_entries, 0);
	assert(additions == 2);
	assert(pdict_ent_remove(pd, "key2", &ov));
	assert(strcmp(ov, "value2") == 0);
	free(ov);
	ov = NULL;
	assert(!pdict_ent_lookup(pd, "key2", &v));

	assert(pdict_remove_persistent_change_listener(pd, id));
	assert(pdict_add(pd, "key3", "val3", NULL));
	assert(additions == 2);

	printf("After removing key2, the dictionary has:\n");
	pdict_walk(pd, print_entries, 0);

#ifndef _WINDOWS
	signal(SIGALRM, sigalrm);
	alarm(period);
	while (!should_exit)
		assert(pdict_add(pd, "key1", "value1b", NULL));
	printf("%d changes/s\n", changes / period);
#endif
	return (0);
}
